#!/usr/bin/env python
from gnuradio import gr, gru, eng_notation, optfir
from gnuradio import usrp
from gnuradio.eng_option import eng_option
import math
import sys
from numpy.numarray import fft
from optparse import OptionParser
from gnuradio.wxgui import stdgui2, fftsink2, scopesink2
import wx

class top_graph(stdgui2.std_top_block):
	def __init__(self, frame, panel, vbox, argv):
		stdgui2.std_top_block.__init__ (self, frame, panel, vbox, argv)
	# ------------------ parameters setting ------------------
		sample_rate = 1e6 # 1MHz
		#usrp_decim = int(64e6 / sample_rate)
		usrp_decim = 112
		antenna_num = 2
		freq_a = 959.4e6
		freq_b = freq_a
		#freq_b = freq_a - 45.0e6
		#freq_b = 957.4e6
		db_gain = 45.0
	# ------------------ USRP init ------------------
		self.u = usrp.source_c (0, usrp_decim) #,fpga_filename="std_2rxhb_2tx.rbf"
		self.u.set_nchannels(antenna_num)
		subdev_a = usrp.selected_subdev(self.u, (0,0)) # (0,0) is A side; (1,0) is B side
		subdev_b = usrp.selected_subdev(self.u, (1,0))
		print "A side:", subdev_a.name()
		print "B side:", subdev_b.name()
		subdev_a.select_rx_antenna('TX/RX')
		subdev_b.select_rx_antenna('TX/RX')
		subdev_a.set_gain(db_gain)
		subdev_b.set_gain(db_gain)
		subdev_a.set_auto_tr(False)
		subdev_b.set_auto_tr(False)

		r = self.u.tune(0, subdev_a, freq_a)
		if not(r):
			print "A side: Failed to set initial frequency"
		else:
			print "A side: The carrier frequency is set as ", freq_a

		r = self.u.tune(1, subdev_b, freq_b)
		if not(r):
			print "B side: Failed to set initial frequency"
		else:
			print "B side: The carrier frequency is set as ", freq_b

		mux_val = self.u.determine_rx_mux_value((0,0),(1,0))
		print "mux_val = 0x%x" % gru.hexint(mux_val)
		self.u.set_mux(mux_val)

		# ------------------ flow graph ------------------
		# s2p
		self.s2p = gr.deinterleave(gr.sizeof_gr_complex)
		# sink
		self.null_sink = gr.null_sink(gr.sizeof_gr_complex)
		self.connect(self.u, self.s2p)
		# fft scope
		self.spectrum_a = fftsink2.fft_sink_c (panel, fft_size=1024,
							sample_rate=sample_rate,title="GSM Downlink on Antenna A")
		self.connect ((self.s2p,0),self.spectrum_a)
		vbox.Add (self.spectrum_a.win, 1, wx.EXPAND)
		self.spectrum_b = fftsink2.fft_sink_c (panel, fft_size=1024,
							sample_rate=sample_rate,title="GSM Uplink on Antenna B")
		self.connect ((self.s2p,1),self.spectrum_b)
		vbox.Add (self.spectrum_b.win, 1, wx.EXPAND)

def main ():
	app = stdgui2.stdapp(top_graph, "2 Antenna Rx")
	app.MainLoop ()

if __name__ == '__main__':
		main ()

